# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['ISANY_SYNONYMS', 'IDENTITY_SYNONYMS', 'isany', 'passable', 'IdentityWarning', 'identity']

# %% ../nbs/00_core.ipynb 4
from typing import (Any, Tuple, Optional)
import warnings
from regsyn import globsyn

# %% ../nbs/00_core.ipynb 6
ISANY_SYNONYMS = {'anyq', 'notnone', 'isthing', 'something', 'isanything',}

# %% ../nbs/00_core.ipynb 7
# @globsyn(ISANY_SYNONYMS, globals())
@globsyn(ISANY_SYNONYMS)
def isany(x, *args, **kwargs) -> bool:
    '''Returns whether or not x is not None.
    
    Parameters
    ----------
    *args : tuple, optional
        Any number of arguments, all of which are returned.        
    **kwargs : dict, optional
        Extra arguments, all of which are returned.

    Notes
    -----
    This function is useful for testing purposes.
    It also is handy as a default callable for methods which require a callable.

    Returns
    -------
    flag : bool
        Whether or not x is not None.

    Examples
    --------
    >>> isany(1)
    >>> True
    '''
    return x != None

# %% ../nbs/00_core.ipynb 9
def passable(*args, **kwargs) -> (Tuple[Any, ...], dict):
    '''Returns all arguments and keyword arguments passed to it as they are.
    
    Parameters
    ----------
    *args : tuple, optional
        Any number of arguments, all of which are returned.        
    **kwargs : dict, optional
        Extra arguments, all of which are returned.

    Notes
    -----
    This function is useful for testing purposes.
    It also is handy as a default callable for methods which require a callable.

    Returns
    -------
    *args : Tuple[Any, ...]
    kwargs : dict

    Examples
    --------
    >>> *args, kwargs = passable(1,2,3, a=2)
    >>> args, kwargs
    >>> ([1, 2, 3], {'a': 2})
    '''
    return *args, kwargs

# %% ../nbs/00_core.ipynb 12
IDENTITY_SYNONYMS = {
    'noc', 'echo', 'iden', 'idfn', 'nada', 'noop', 'nodo', 'same',
    'nomod', 'idfunc', 'nocall', 'mirror', 'reflect', 'giveback', 'keepsame', 'donothing'
}

# %% ../nbs/00_core.ipynb 14
class IdentityWarning(UserWarning):
    '''A warning to be raised when an identity function is called with arguments.
    
    Attributes
    ----------
    source : Optional[str]
        The source where the warning originated.
    stcklvl : int
        The stack level for the warning.
    warn : bool
        Whether to actually issue a warning.
    n_args : int
        Number of positional arguments received.
    n_kwds : int
        Number of keyword arguments received.
    
    Methods
    -------
    makemsg()
        Generate the warning message.
    __call__()
        Issue the warning if applicable.
    '''
    
    source: Optional[str] = None
    stcklvl: int = 3

    def __init__(self, *args, **kwargs):
        super().__init__(*args)
        
        self.warn = kwargs.pop('warn', False)
        self.n_args = len(args)
        self.n_kwds = len(kwargs)
    
    def makemsg(self) -> str:
        '''Generate the warning message based on the number of arguments and keyword arguments.'''
        parts = []
        
        if self.n_args:
            parts.append(f'The identity function does not expect args but received `{self.n_args}` *args.')
        
        if self.n_kwds:
            parts.append(f'The identity function does not expect kwargs but received `{self.n_kwds}` **kwargs.')
        
        return ' '.join(parts)

    @property
    def dowarn(self) -> bool:
        '''Determine whether a warning should be issued.'''
        return self.warn and (self.n_args or self.n_kwds)

    def __call__(self) -> None:
        '''Issue the warning if applicable.'''
        if self.dowarn: 
            msg = self.makemsg()
            warnings.warn(msg, type(self), stacklevel=self.stcklvl, source=self.source)


# %% ../nbs/00_core.ipynb 16
@globsyn(IDENTITY_SYNONYMS)
def identity(x: Any, *args, **kwargs) -> Any:
    '''Identity function
    
    Parameters
    ----------
    x : Any
        Any object

    Other Parameters
    ----------------
    *args : tuple, optional
        Any number of arguments, all of which are ignored.        
    **kwargs : dict, optional
        Extra arguments, all of which are ignored.
    
    Returns
    -------
    Any

    Notes
    -----
    This function is useful for testing purposes.
    It also is handy as a default callable for methods which require a callable.

    See Also
    --------
    noc : Alias for identity (No Callable)
    echo : Alias for identity
    iden : Alias for identity
    idfn : Alias for identity (IDentity FuNction)
    nada : Alias for identity
    noop : Alias for identity (NO OPeration)
    nodo : Alias for identity (NO DO)
    same : Alias for identity
    nomod : Alias for identity (NO MODification)
    idfunc : Alias for identity (IDentity FUNCtion)
    nocall : Alias for identity (NO CALLable)
    mirror : Alias for identity 
    reflect : Alias for identity
    giveback : Alias for identity
    keepsame : Alias for identity
    donothing : Alias for identity
    
    Examples
    --------
    >>> identity(1)
    1
    >>> identity('a')
    'a'
    >>> identity([1,2,3])
    [1, 2, 3]
    '''
    IdentityWarning(*args, **kwargs)()
    return x
